--[[
****************************************************************************************
Welfare Epics
$Date: 2008-07-07 00:27:15 -0400 (Mon, 07 Jul 2008) $
$Rev: 77962 $

Author: Ackis on Illidan US Horde
****************************************************************************************

Please see Wowace.com for more information.

****************************************************************************************
]]


local L			= LibStub("AceLocale-3.0"):GetLocale("WelfareEpics")
local IBL		= LibStub("LibItemBonus-2.0")

WelfareEpics	= LibStub("AceAddon-3.0"):NewAddon("WelfareEpics", "AceConsole-3.0", "AceEvent-3.0", "AceHook-3.0")

local addon = WelfareEpics

local GetMerchantItemLink = GetMerchantItemLink
local GetMerchantItemInfo = GetMerchantItemInfo

-- Returns main description configuration
local function giveOptions()

	local options =
	{
		desc =
		{
			order = 1,
			type = "description",
			name = L["FILTER_OPTIONS"] .. "\n",
		},
	}

	return options

end

-- Returns configuration options for profiling
local function giveProfiles()

	local profiles = LibStub("AceDBOptions-3.0"):GetOptionsTable(addon.db)
	return profiles

end

-- Returns configuraion options for filter
local function giveFilter()

	local filter =
	{
		desc =
		{
			order = 1,
			type = "description",
			name = L["FILTER_OPTIONS"] .. "\n",
		},
		factions =
		{
			name	= L["Usable"],
			desc	= L["USABLE_TOGGLE"],
			type	= "toggle",
			get		= function() return addon.db.profile.usable end,
			set		= function() addon.db.profile.usable = not addon.db.profile.usable end,
			order	= 2,
		},
	}

end

-- Register slash commands and profile defaults
function addon:OnInitialize()

	local AceConfigReg = LibStub("AceConfigRegistry-3.0")
	local AceConfigDialog = LibStub("AceConfigDialog-3.0")

	self.db = LibStub("AceDB-3.0"):New("AckisRecipeListDB",defaults,"Default")

	-- Create the options with Ace3
	LibStub("AceConfig-3.0"):RegisterOptionsTable("Welfare Epics",giveOptions,"Welfare Epics")
	AceConfigReg:RegisterOptionsTable("Welfare Epics Filter",giveFilter)
	AceConfigReg:RegisterOptionsTable("Welfare Epics Profile",giveProfiles)

	-- Add the options to blizzard frame (add them backwards so they show up in the proper order
	self.optionsFrame = AceConfigDialog:AddToBlizOptions("Welfare Epics","Welfare Epics")
	self.optionsFrame[L["Profile"]] = AceConfigDialog:AddToBlizOptions("Welfare Epics Profile", L["Profile"], "Welfare Epics")
	self.optionsFrame[L["Filter"]] = AceConfigDialog:AddToBlizOptions("Welfare Epics Filter", L["Filter"], "Welfare Epics")

	-- Register slash commands
	self:RegisterChatCommand("welfareepics", "ChatCommand")

	-- Set default options, which are to include everything in the scan
	self.db:RegisterDefaults(
		{ profile =
			{
			-- Filter Options
		    usable = true,
			}
		}
	)

end

-- Register events and hooks
function addon:OnEnable()

	-- Register events
	self:RegisterEvent("MERCHANT_SHOW")
	self:RegisterEvent("MERCHANT_CLOSED")

	-- Hook into the merchant frame update running code after it is run
	self:RawHook("MerchantFrame_Update", "MerchantFrame_Update", true)
	--self:RawHook("MerchantFrame_UpdateMerchantInfo", "HideUnusable", true)

	PanelTemplates_SetNumTabs(MerchantFrame, 4)

	self:Print("This mod is unstable.  Please don't it until there is a forum post.  Thank you, Ackis")

end

do

	-- List of indexes of all items availible to the vendor
	local vendoritems = nil
	local merchantlist = {}
	merchantlist["Cloth"] = nil
	merchantlist["Leather"] = {}
	merchantlist["Mail"] = {}
	merchantlist["Plate"] = {}
	merchantlist["Weapon"] = {}
	merchantlist["Other"] = {}

	-- Scans the vendor list adding item indexes to a local table
	function addon:MERCHANT_SHOW()

		self:ParseItems()
		--self:SetupNewTabs()

	end

	-- Nils out the table
	function addon:MERCHANT_CLOSED()

		vendoritems = nil

		-- Reset the type tables
		merchantlist["Cloth"] = nil
		merchantlist["Leather"] = {}
		merchantlist["Mail"] = {}
		merchantlist["Plate"] = {}
		merchantlist["Weapon"] = {}
		merchantlist["Other"] = {}

	end

	-- Scans the availible items at a vendor
	function addon:ParseItems()

		-- Make sure our tables are created
		if not vendoritems then
			vendoritems = {}
		end

		if not merchantlist["Cloth"] then
			merchantlist["Cloth"] = {}
		end

		local numMerchantItems = GetMerchantNumItems()
		for i=1,numMerchantItems,1 do
			-- See if the item is usable or not
			local _, _, _, _, _, isUsable = GetMerchantItemInfo(i)
			-- Get the item ID
			--local _, _, itemID = string.find(GetMerchantItemLink(i), "^|c%x+|Hitem:([0-9]+)[:0-9]+|h%[.+%]")
			-- Attempt to categorize the item
			local itemtype,classtype = addon:ScanLink(GetMerchantItemLink(i))
			table.insert(merchantlist[itemtype],i)
			-- Get a list of usable items
			if isUsable then
				table.insert(vendoritems,i)
			end
		end

	end

	function addon:AddFilteredMerchantItems(filtertype)

		--local numMerchantItems = GetMerchantNumItems()
		local numMerchantItems = #merchantlist[filtertype]

		for i=1, MERCHANT_ITEMS_PER_PAGE, 1 do
			local vendorcount = (i + ((MerchantFrame.page - 1) * MERCHANT_ITEMS_PER_PAGE))
			local index = merchantlist[filtertype][vendorcount]
			-- If we're at empty spots (ie: no items to display) add an empty item
			if vendorcount > numMerchantItems then
				addon:AddBlankVendorItem(i)
			-- If we still have items to display, add that items
			else
				addon:AddMerchantItem(index,i,numMerchantItems)
			end
		end

	end

	function addon:MerchantFrame_Update()
		if (MerchantFrame.selectedTab == 1) then
			--MerchantFrame_UpdateMerchantInfo()
			self:HideUnusable()
		elseif (MerchantFrame.selectedTab == 2) then
			MerchantFrame_UpdateBuybackInfo()
		elseif (MerchantFrame.selectedTab == 3) then
			self:AddFilteredMerchantItems("Cloth")
		end
	end

	-- Same as blizzard code but references the table listing indexes of usable items only
	function addon:HideUnusable()

		-- If for some reason we don't have the index built yet, lets build it
		if not vendoritems then
			self:ParseItems()
		end

		MerchantNameText:SetText(UnitName("NPC"))
		SetPortraitTexture(MerchantFramePortrait, "NPC")

		--local numMerchantItems = GetMerchantNumItems()
		local numMerchantItems = #vendoritems
	 
		MerchantPageText:SetFormattedText(MERCHANT_PAGE_NUMBER, MerchantFrame.page, math.ceil(numMerchantItems / MERCHANT_ITEMS_PER_PAGE))

		for i=1, MERCHANT_ITEMS_PER_PAGE, 1 do
			local vendorcount = (i + ((MerchantFrame.page - 1) * MERCHANT_ITEMS_PER_PAGE))
			local index = vendoritems[vendorcount]
			-- If we're at empty spots (ie: no items to display) add an empty item
			if vendorcount > numMerchantItems then
				addon:AddBlankVendorItem(i)
			-- If we still have items to display, add that items
			else
				addon:AddMerchantItem(index,i,numMerchantItems)
			end
		end

		addon:HandleBlizzStuff(numMerchantItems)

		-- Show all merchant related items
		MerchantBuyBackItem:Show()
		MerchantFrameBottomLeftBorder:Show()
		MerchantFrameBottomRightBorder:Show()
		 
		-- Hide buyback related items
		MerchantItem11:Hide()
		MerchantItem12:Hide()
		BuybackFrameTopLeft:Hide()
		BuybackFrameTopRight:Hide()
		BuybackFrameBotLeft:Hide()
		BuybackFrameBotRight:Hide()
		 
		-- Position merchant items
		MerchantItem3:SetPoint("TOPLEFT", "MerchantItem1", "BOTTOMLEFT", 0, -8)
		MerchantItem5:SetPoint("TOPLEFT", "MerchantItem3", "BOTTOMLEFT", 0, -8)
		MerchantItem7:SetPoint("TOPLEFT", "MerchantItem5", "BOTTOMLEFT", 0, -8)
		MerchantItem9:SetPoint("TOPLEFT", "MerchantItem7", "BOTTOMLEFT", 0, -8)

	end

	-- Adds blank item to vendor
	function addon:AddBlankVendorItem(VendorItemLocation)
		local merchantButton = getglobal("MerchantItem"..VendorItemLocation)
		local itemButton = getglobal("MerchantItem"..VendorItemLocation.."ItemButton")

		itemButton.hasItem = nil
		itemButton:Hide()
		SetItemButtonNameFrameVertexColor(merchantButton, 0.5, 0.5, 0.5)
		SetItemButtonSlotVertexColor(merchantButton,0.4, 0.4, 0.4)
		getglobal("MerchantItem"..VendorItemLocation.."Name"):SetText("")
		getglobal("MerchantItem"..VendorItemLocation.."MoneyFrame"):Hide()
		getglobal("MerchantItem"..VendorItemLocation.."AltCurrencyFrame"):Hide()
	end

	-- The reamining code that blizzard has, deals with buybacks and paging buttons
	function addon:HandleBlizzStuff(numMerchantItems)

		-- Handle repair items
		MerchantFrame_UpdateRepairButtons()
		 
		-- Handle vendor buy back item
		local buybackName, buybackTexture, buybackPrice, buybackQuantity, buybackNumAvailable, buybackIsUsable = GetBuybackItemInfo(GetNumBuybackItems())
		if ( buybackName ) then
			MerchantBuyBackItemName:SetText(buybackName)
			SetItemButtonCount(MerchantBuyBackItemItemButton, buybackQuantity)
			SetItemButtonStock(MerchantBuyBackItemItemButton, buybackNumAvailable)
			SetItemButtonTexture(MerchantBuyBackItemItemButton, buybackTexture)
			MerchantBuyBackItemMoneyFrame:Show()
			MoneyFrame_Update("MerchantBuyBackItemMoneyFrame", buybackPrice)
			MerchantBuyBackItem:Show()
		 
		else
			MerchantBuyBackItemName:SetText("")
			MerchantBuyBackItemMoneyFrame:Hide()
			SetItemButtonTexture(MerchantBuyBackItemItemButton, "")
			SetItemButtonCount(MerchantBuyBackItemItemButton, 0)
		-- Hide the tooltip upon sale
			if ( GameTooltip:IsOwned(MerchantBuyBackItemItemButton) ) then
				GameTooltip:Hide()
			end
		end
		 
		-- Handle paging buttons
		if ( numMerchantItems > MERCHANT_ITEMS_PER_PAGE ) then
			if ( MerchantFrame.page == 1 ) then
				MerchantPrevPageButton:Disable()
			else
				MerchantPrevPageButton:Enable()
			end
			if ( MerchantFrame.page == ceil(numMerchantItems / MERCHANT_ITEMS_PER_PAGE) or numMerchantItems == 0) then
				MerchantNextPageButton:Disable()
			else
				MerchantNextPageButton:Enable()
			end
			MerchantPageText:Show()
			MerchantPrevPageButton:Show()
			MerchantNextPageButton:Show()
		else
			MerchantPageText:Hide()
			MerchantPrevPageButton:Hide()
			MerchantNextPageButton:Hide()
		end
		 
	end

	-- Adds an item to the vendor.  It is venodr ID VendorItemIndex and put into location VendorItemLocation
	function addon:AddMerchantItem(VendorItemIndex,VendorItemLocation,numMerchantItems)

		local name, texture, price, quantity, numAvailable, isUsable, extendedCost

		local itemButton = getglobal("MerchantItem"..VendorItemLocation.."ItemButton")
		local merchantButton = getglobal("MerchantItem"..VendorItemLocation)
		local merchantMoney = getglobal("MerchantItem"..VendorItemLocation.."MoneyFrame")

		local merchantAltCurrency = getglobal("MerchantItem"..VendorItemLocation.."AltCurrencyFrame")
		--if ( VendorItemIndex <= numMerchantItems ) then
			name, texture, price, quantity, numAvailable, isUsable, extendedCost = GetMerchantItemInfo(VendorItemIndex)
			getglobal("MerchantItem"..VendorItemLocation.."Name"):SetText(name)
			SetItemButtonCount(itemButton, quantity)
			SetItemButtonStock(itemButton, numAvailable)
			SetItemButtonTexture(itemButton, texture)
	 
			if ( extendedCost and (price <= 0) ) then
				itemButton.extendedCost = true
				itemButton.link = GetMerchantItemLink(VendorItemIndex)
				itemButton.texture = texture
				MerchantFrame_UpdateAltCurrency(VendorItemIndex, VendorItemLocation)
				merchantAltCurrency:ClearAllPoints()
				merchantAltCurrency:SetPoint("BOTTOMLEFT", "MerchantItem"..VendorItemLocation.."NameFrame", "BOTTOMLEFT", 0, 31)
				merchantMoney:Hide()
				merchantAltCurrency:Show()
			elseif ( extendedCost and (price > 0) ) then
				itemButton.extendedCost = true
				itemButton.link = GetMerchantItemLink(VendorItemIndex)
				itemButton.texture = texture
				MerchantFrame_UpdateAltCurrency(VendorItemIndex, VendorItemLocation)
				MoneyFrame_Update(merchantMoney:GetName(), price)
				merchantAltCurrency:ClearAllPoints()
				merchantAltCurrency:SetPoint("LEFT", merchantMoney:GetName(), "RIGHT", -14, 0)
				merchantAltCurrency:Show()
				merchantMoney:Show()
			else
				itemButton.extendedCost = nil
				MoneyFrame_Update(merchantMoney:GetName(), price)
				merchantAltCurrency:Hide()
				merchantMoney:Show()
			end
 
			itemButton.hasItem = true
			itemButton:SetID(VendorItemIndex)
			itemButton:Show()

			if ( numAvailable == 0 ) then
				-- If not available and not usable
				if ( not isUsable ) then
					SetItemButtonNameFrameVertexColor(merchantButton, 0.5, 0, 0)
					SetItemButtonSlotVertexColor(merchantButton, 0.5, 0, 0)
					SetItemButtonTextureVertexColor(itemButton, 0.5, 0, 0)
					SetItemButtonNormalTextureVertexColor(itemButton, 0.5, 0, 0)
				else
					SetItemButtonNameFrameVertexColor(merchantButton, 0.5, 0.5, 0.5)
					SetItemButtonSlotVertexColor(merchantButton, 0.5, 0.5, 0.5)
					SetItemButtonTextureVertexColor(itemButton, 0.5, 0.5, 0.5)
					SetItemButtonNormalTextureVertexColor(itemButton,0.5, 0.5, 0.5)
				end
	 			elseif ( not isUsable ) then
				SetItemButtonNameFrameVertexColor(merchantButton, 1.0, 0, 0)
				SetItemButtonSlotVertexColor(merchantButton, 1.0, 0, 0)
				SetItemButtonTextureVertexColor(itemButton, 0.9, 0, 0)
				SetItemButtonNormalTextureVertexColor(itemButton, 0.9, 0, 0)
			else
				SetItemButtonNameFrameVertexColor(merchantButton, 0.5, 0.5, 0.5)
				SetItemButtonSlotVertexColor(merchantButton, 1.0, 1.0, 1.0)
				SetItemButtonTextureVertexColor(itemButton, 1.0, 1.0, 1.0)
				SetItemButtonNormalTextureVertexColor(itemButton, 1.0, 1.0, 1.0)
			end
	end

end

-- Scans a given item link outputting the type of armour/weapon it is, and what type of piece (melee, caster, healer, tank)
function addon:ScanLink(link)

	local _, _, _, _, _, itemType, itemSubType, _, itemEquipLoc = GetItemInfo(link)
	--ScanItemLink
	if (itemType == "Armor") then
		if (itemSubType == "Cloth") then
			return "Cloth","Caster DPS"
		elseif (itemSubType == "Leather") then
			return "Leather","Caster DPS"
		elseif (itemSubType == "Mail") then
			return "Mail","Caster DPS"
		elseif (itemSubType == "Plate") then
			return "Plate","Caster DPS"
		else
			return "Other","Caster DPS"
		end
	elseif (itemType == "Weapon") then
		return "Weapon","Caster DPS"
	else
		return "Other","Caster DPS"
	end

end

-- Adds new tabs to the vendor windows for specific types of items
function addon:SetupNewTabs()

	addon.NewTab1 = CreateFrame("Button","MerchantFrameTab3",MerchantFrameTab2,"CharacterFrameTabButtonTemplate")
	addon.NewTab1:SetText("Cloth")
	addon.NewTab1:ClearAllPoints()
	addon.NewTab1:SetPoint("LEFT",MerchantFrameTab2,"RIGHT",-16,0)
	addon.NewTab1:SetScript("OnEnter", function(this)
										GameTooltip:SetOwner(this, "ANCHOR_RIGHT")
										GameTooltip:SetText("", 1.0,1.0,1.0)
									end)
	addon.NewTab1:SetScript("OnLeave", function(this)
										GameTooltip:Hide()
									end)
	addon.NewTab1:SetScript("OnClick", function(this)
										PanelTemplates_SetTab(MerchantFrame, this:GetID())
										MerchantFrame_Update()
									end)
	addon.NewTab1:SetID(3)

	addon.NewTab2 = CreateFrame("Button","MerchantFrameTab4",MerchantFrameTab3,"CharacterFrameTabButtonTemplate")
	addon.NewTab2:SetText("Leather")
	addon.NewTab2:ClearAllPoints()
	addon.NewTab2:SetPoint("LEFT",MerchantFrameTab3,"RIGHT",-16,0)
	addon.NewTab2:SetScript("OnEnter", function(this)
										GameTooltip:SetOwner(this, "ANCHOR_RIGHT")
										GameTooltip:SetText("", 1.0,1.0,1.0)
									end)
	addon.NewTab2:SetScript("OnLeave", function(this)
										GameTooltip:Hide()
									end)
	addon.NewTab2:SetScript("OnClick", function(this)
										PanelTemplates_SetTab(MerchantFrame, this:GetID())
										MerchantFrame_Update()
									end)
	addon.NewTab2:SetID(4)

end

